# file      : build/cxx/intel/cxx-o.make
# author    : Boris Kolpackov <boris@kolpackov.net>
# copyright : Copyright (c) 2004 Boris Kolpackov
# license   : GNU GPL v2; see accompanying LICENSE file

$(call include-once,$(bld_root)/c/cpp-options.make,$(out_root))
$(call include,$(bld_root)/cxx/intel/configuration.make)

# Make will try to build dependecies (since they are ultimately included
# files) during configuartion phase without cxx_intel being discovered yet.
# This is also why dependecies should be included with -include.
#
ifdef cxx_intel

$(out_root)/%.o.d: cxx := $(cxx_intel)
$(out_root)/%.o.d: cxx_options +=

#@@ This needs to be shared with cxx-o.
#
#@@ wrong prefix
#
define expand-cpp-options-impl
$(if $1,$(shell sed -e 's%include: \(.*\)%\1%;t;d' $1))
endef

define expand-cpp-options
$(call expand-cpp-options-impl,$(filter %.cpp-options,$1))
endef


# Makes prefix-appending sed script from cpp-options' prefix: lines.
#
define make-prefix-script-impl
$(if $1,$(shell sed -e 's%prefix: \([^ ]*\) \([^ ]*\)%s \\ \1 \\ \2\1 g;s ^\1 \2\1 ;%;t;d' $1))
endef

define make-prefix-script
$(call make-prefix-script-impl,$(filter %.cpp-options,$1))
endef


define make-sed-command
| sed -e '$(if $(strip $1),$1)s% \([^/ \\]\)% $(out_base)/\1%g;s%^\([^/ ].*:\)%$(out_base)/\1%' >$2
endef


.PRECIOUS: $(out_root)/%.o.d

$(out_root)/%.o.d: $(out_root)/%$(cxx_s_suffix) | $$(dir $$@).
	$(call message,,if test -f $@; then mv $@ $@.old; fi && \
$(cxx) $(cpp_options) $(call expand-cpp-options,$^) \
-M -MG -MP -MQ $@ -MQ $(basename $@) $< \
$(call make-sed-command,$(call make-prefix-script,$^),$@) && \
if test -f $@.old; then if cmp -s $@ $@.old; then rm -f $@.old && false; \
else rm -f $@.old && true; fi fi)


ifneq ($(out_root),$(src_root))

$(out_root)/%.o.d: $(src_root)/%$(cxx_s_suffix) | $$(dir $$@).
	$(call message,,if test -f $@; then mv $@ $@.old; fi && \
$(cxx) $(cpp_options) $(call expand-cpp-options,$^) \
-M -MG -MP -MQ $@ -MQ $(basename $@) $< \
$(call make-sed-command,$(call make-prefix-script,$^),$@) && \
if test -f $@.old; then if cmp -s $@ $@.old; then rm -f $@.old && false; \
else rm -f $@.old && true; fi fi)

endif


.PHONY: $(out_root)/%.o.d.clean

$(out_root)/%.o.d.clean:
	$(call message,rm $(basename $@),rm -f $(basename $@))

endif
